% This is the main run file for estimating the regime switching skew-normal
% model in Chan, Fry-McKibbin and Hsiao (2018)
%
% This code is free to use for academic purposes only, provided that the 
% paper is cited as:
%
% Chan, J.C.C, Fry-McKibbin, R. A. and Hsiao, C. Y.-L. (2018). A Regime 
% Switching Skew-normal Model of Contagion, Studies in Nonlinear Dynamics
% and Econometrics, forthcoming
%
% This code comes without technical support of any kind.  It is expected to
% reproduce the results reported in the paper. Under no circumstances will
% the authors be held responsible for any use (or misuse) of this code in
% any way.

%==========================================================================
% The data is in USD currency, from 3 January 2005 to 28 November 2014.
% Non-crisis period are defined from 1 January 2005 to 30 June 2007.
% Crisis period are defined from 1 March 2008 to 28 November 2014
% Data presented in the following order:

                       % Equity Price Index
                       % (source: Datastream for 5 countries )
    
                      % 1) France                  FRCAC40(PI)~U$
                      % 2) Germany                 MDAXIDX(PI)~U$
                      % 3) Greece                  GRAGENL(PI)~U$
                      % 4) Italy                   FTSEMIB(PI)~U$
                      % 5) US                      DJINDUS(PI)~U$
                      
%==========================================================================

clear; clc;
nloop = 200000;
burnin = 10000;

% load the data
equity = xlsread('equityindices.xlsx','Sheet1','B6:f2590');
e_eqt = 100*(log(equity(2:end,:))-log(equity(1:end-1,:))); %equity returns (2584 obs)

% Non-crisis period is from January 4, 2005 to July 25, 2007 
% crisis period is from July 26, 2007 to November 28, 2014
xt = e_eqt(1:667,:);     
yt = e_eqt(668:2584,:); 

%full sample period
shortY = e_eqt;  
[N m] = size(shortY);
Y = reshape(shortY',N*m,1);  
c = -sqrt(2/pi); 

% initialize for storage
tempid = reshape(1:m^2,m,m);
tempA = triu(tempid)';   idDel = tempA(tempA~=0);
tempA = triu(tempid,1);  idu = tempA(tempA~=0);
tempA = tril(tempid,-1)'; idl = tempA(tempA~=0);
k = m*(m+1)/2;
sbeta1 = zeros(nloop-burnin,m+k);
sSig1 = zeros(nloop-burnin,k);
sbeta2 = zeros(nloop-burnin,m+k);
sSig2 = zeros(nloop-burnin,k);
sCorr1 = zeros(nloop-burnin,(m*(m-1))/2);
sCorr2 = zeros(nloop-burnin,(m*(m-1))/2);
sS = zeros(N,1);
sigj = zeros(1,m-1); % the jth row of Sig minus the jth element
invSigjj = zeros(m-1,m-1); % m-1 by m-1 minor of Sigma by deleting the jth row and column,
like = zeros(N,2);
slpost_samebeta = zeros(nloop-burnin,1);
slpost_samemu = zeros(nloop-burnin,1);
slpost_samedel = zeros(nloop-burnin,1);
slpost_samskew = zeros(nloop-burnin,1);
slpost_samecoskew = zeros(nloop-burnin,1);
slpost_samebetai = zeros(nloop-burnin,m+k);
siglpost_sigs = zeros(k,1);

% prior
beta0 = zeros(k+m,1);
Vbeta = diag([.1^2*ones(1,m) ones(1,k)]);
invVbeta = Vbeta\speye(k+m);
nuSig0 = 20 + m + 1;
Vsig0 = (nuSig0-m-1)*speye(m);
p0 = zeros(N,1);
p0(1:649) = 0.9999; % Before June 30, 2007 (non-crisis date)
p0(650:824) = linspace(.9999,.0001,824-650+1);  
p0(825:end) = 0.0001;  % After March 03, 2008 (crisis date)
    
% initialize
cc = 5; %source crisis market (the US)
nx = size(xt,1);
ny = size(yt,1);
S = [ones(nx,1); 2*ones(N-nx,1);];
id1 = find(S==1);   id2 = find(S==2);
n1 = length(id1);   n2 = N - n1;
mu1 = mean(shortY(id1,:))';
stdY1 = std(shortY(id1,:))';
e = shortY(id1,:) - repmat(mu1',n1,1);
Del1 = (e.^2'*e/n1)./(stdY1.^2*stdY1');
Del1 = (Del1 + Del1')/2;
beta1 = [mu1; Del1(idDel)];   
Sig1 = cov(shortY(id1,:));
invSig1 = Sig1\speye(m);
mu2 = mean(shortY(id2,:))';
stdY2 = std(shortY(id2,:))';
e = shortY(id2,:) - repmat(mu2',n2,1);
Del2 = (e.^2'*e/n2)./(stdY2.^2*stdY2');
Del2 = (Del2 + Del2')/2;
beta2 = [mu2; Del2(idDel)];
Sig2 = cov(shortY(S==2,:));
invSig2 = Sig2\speye(m);
id1 = find(S==1);   id2 = find(S==2);
n1 = length(id1);   n2 = N - n1;
shortY1 = shortY(id1,:);         shortY2 = shortY(id2,:);
Y1 = reshape(shortY1',m*n1,1);    Y2 = reshape(shortY2',m*n2,1);
Z = initialZ(S,shortY,Del1,mu1,Del2,mu2,invSig1,invSig2);

count = 1; %the location for variance and co-variance parameter (square matrix)
Sigid = zeros(m*(m+1)/2,1);
for i=1:m
    for j=i:m
        Sigid(count) = (i-1)*m+j;
        count = count+1;
    end    
end

count = 1;    %the location for coskewness parameter (square matrix)
Sigid_coskew = zeros(m*(m-1)/2,1);
for i=1:(m-1)
    for j=i:(m-1)
        Sigid_coskew(count) = (i-1)*(m)+(j+1);
        count = count + 1;
    end    
end
    
jj = m:-1:1;   %the location for variance parameter (triangular matrix)
kk_loc = zeros(m,1);
kk_loc(1,1) = 1;
for ii = 2:m
    kk_loc(ii,1) = kk_loc(ii-1,1)+ jj(1,ii-1);
end

subm = m-1;  %the location for variance parameter (triangular matrix without leading diagonal)
jjj = subm:-1:1;  
kk_subloc = zeros(subm,1);
kk_subloc(1,1) = 1;
for ii = 2:subm
kk_subloc(ii,1) = kk_subloc(ii-1,1)+ jjj(1,ii-1);
end

Vcoskew_loc = zeros((m*(m-1))/2,1)'; %the location for coskewness parameter (triangular matrix)
 for i = 1:m-1
  kk_loc(m) = ((m*(m+1))/2);
  Vcoskew_ele = (kk_loc(i)+1):(kk_loc(i+1)-1);              
  kk_subloc(m) = (((m-1)*(m))/2)+1;
  Vcoskew_loc(kk_subloc(i):kk_subloc(i+1)-1)=Vcoskew_ele;
 end
 Vcoskew_loc = Vcoskew_loc';

rand('state', sum(1000*clock) );    randn('state', sum(5000*clock) );
disp('Estimating the RSSN model using MCMC.... ');
start_time = clock;

% Estimation
for loop = 1 : nloop  
       
 %% sample Z
  DelinvSig1 = Del1'*invSig1;
  Dz1 = (speye(m) + DelinvSig1*Del1)\speye(m); % common covariance for all Z_i's 
  dz1 = c + DelinvSig1*(shortY1' - repmat(mu1,1,n1));
  Zmu1 = (Dz1*dz1)';
  for j = 1:m        
    sigj = Dz1(j,[1:j-1 j+1:end]);
    invSigjj = Dz1([1:j-1 j+1:end], [1:j-1 j+1:end])\speye(m-1);
    Zerr = Z(id1,:) - Zmu1;
    Zconmu = Zerr(:,[1:j-1 j+1:end]);            
    muj = Zmu1(:,j) + (sigj*invSigjj*Zconmu')'; % compute the condistional mean
    vj = Dz1(j,j) - sigj*invSigjj*sigj';        
    % check if there're any inf values
    Zj = Z(id1,j);
    tempZ = tnormrnd(muj,vj*ones(n1,1),c,inf,n1);
    id = find(isinf(tempZ)==1);
    tempZ(id) = Zj(id);
    Z(id1,j) = tempZ; 
  end    
  DelinvSig2 = Del2'*invSig2;
  Dz2 = (speye(m) + DelinvSig2*Del2)\speye(m);
  dz2 = c + DelinvSig2*(shortY2' - repmat(mu2,1,n2));
  Zmu2 = (Dz2*dz2)';    
  for j = 1:m        
    sigj = Dz2(j,[1:j-1 j+1:end]);
    invSigjj = Dz2([1:j-1 j+1:end], [1:j-1 j+1:end])\speye(m-1);
    Zerr = Z(id2,:) - Zmu2;
    Zconmu = Zerr(:,[1:j-1 j+1:end]);            
    muj = Zmu2(:,j) + (sigj*invSigjj*Zconmu')'; % compute the condistional mean
    vj = Dz2(j,j) - sigj*invSigjj*sigj';        
    % check if there're any inf values
    Zj = Z(id2,j);
    tempZ = tnormrnd(muj,vj*ones(n2,1),c,inf,n2);
    id = find(isinf(tempZ)==1);
    tempZ(id) = Zj(id);
    Z(id2,j) = tempZ;
  end     
    
 %% sample S      
  bigX = [repmat(speye(m),N,1) makebigX(Z)];
  like(:,1) = mvnpdf(shortY,reshape(bigX*beta1,m,N)',Sig1); 
  like(:,2) = mvnpdf(shortY,reshape(bigX*beta2,m,N)',Sig2); 
  prS = like.*[p0 1-p0];
  prS = prS./repmat(sum(prS,2),1,2);
  tempdraw = rand(N,1);    
  S = 3-sum(cumsum(prS,2)> repmat(tempdraw,1,2),2);   
    
%% sample Sigma    
  id1 = find(S==1);   id2 = find(S~=1);
  n1 = length(id1);   n2 = N - n1;
  shortY1 = shortY(id1,:);         shortY2 = shortY(id2,:);
  Y1 = reshape(shortY1',m*n1,1);    Y2 = reshape(shortY2',m*n2,1);
  bigX1 = [repmat(speye(m),n1,1) makebigX(Z(id1,:))];
  bigX2 = [repmat(speye(m),n2,1) makebigX(Z(id2,:))];
  u1 = shortY1' - reshape(bigX1*beta1,m,n1); 
  newS1 = Vsig0 + u1*u1';
  Sig1 = iwishrnd(newS1,nuSig0+n1);
  invSig1 = Sig1\speye(m);
  u2 = shortY2' - reshape(bigX2*beta2,m,n2); 
  newS2 = Vsig0 + u2*u2';
  Sig2 = iwishrnd(newS2,nuSig0+n2);
  invSig2 = Sig2\speye(m);    
    
% calculate correlation
  var1 = diag(Sig1);
  denu1_corr = zeros(m,m);
  for i = 1:m
      denu1_corr(:,i) = sqrt(var1(i,1))*sqrt(var1);
  end
  cor1 = Sig1./denu1_corr;
  corr1 = cor1(Sigid_coskew);
  var2 = diag(Sig2);
  denu2_corr = zeros(m,m);
  for i = 1:m
      denu2_corr(:,i) = sqrt(var2(i,1))*sqrt(var2);
  end
  cor2 = Sig2./denu2_corr;
  corr2 = cor2(Sigid_coskew);
    
%% sample beta
  sumtemp1 = bigX1'*kron(speye(n1),invSig1);
  invDb1 = invVbeta + sumtemp1*bigX1;
  db1 = sumtemp1*Y1;
  beta1 = invDb1\db1 + chol(invDb1)\randn(k+m,1);
  mu1 = beta1(1:m,1);
  Del1(idDel) = beta1(m+1:end);
  Del1(idl) = Del1(idu);      
  sumtemp2 = bigX2'*kron(speye(n2),invSig2);
  invDb2 = invVbeta + sumtemp2*bigX2;
  db2 = sumtemp2*Y2;
  beta2 = invDb2\db2 + chol(invDb2)\randn(k+m,1);
  mu2 = beta2(1:m,1);
  Del2(idDel) = beta2(m+1:end);
  Del2(idl) = Del2(idu);    

%% compute the SD density ratio
  Db1 = invDb1\speye(k+m);
  Db2 = invDb2\speye(k+m);
  beta1hat = invDb1\db1;
  beta2hat = invDb2\db2;
        
 % same beta
  gamhat = beta1hat - beta2hat;
  gamvar = Db1 + Db2;
  lpost_samebeta = -.5*(k+m)*log(2*pi) - sum(log(diag(chol(gamvar)))) + ...
                   -.5*gamhat'*(gamvar\gamhat);
            
 % same mu
  gam_muhat = beta1hat(1:m) - beta2hat(1:m);
  gam_muvar = Db1(1:m,1:m) + Db2(1:m,1:m);
  lpost_samemu = -.5*m*log(2*pi) - sum(log(diag(chol(gam_muvar)))) + ...
                 -.5*gam_muhat'*(gam_muvar\gam_muhat);
        
 % same delta
  gam_delhat = beta1hat(m+1:end) - beta2hat(m+1:end);
  gam_delvar = Db1(m+1:end,m+1:end) + Db2(m+1:end,m+1:end);
  lpost_samedel = -.5*k*log(2*pi) - sum(log(diag(chol(gam_delvar)))) + ...
                  -.5*gam_delhat'*(gam_delvar\gam_delhat);
        
 % same skewness
  gam_skewhat = gam_delhat(kk_loc);
  gam_skewvar = eye(m);
  for ss = 1:m
   gam_skewvar(ss,ss) = gam_delvar(kk_loc(ss),kk_loc(ss)); 
  end
  lpost_samskew = -.5*m*log(2*pi) - sum(log(diag(chol(gam_skewvar)))) + ...
                      -.5*gam_skewhat'*(gam_skewvar\gam_skewhat);
        
 % same coskewness not include skewness
  gam_coskewhat = gam_delhat(Vcoskew_loc);
  gam_delcovar = Db1(Sigid_coskew,Sigid_coskew) + Db2(Sigid_coskew,Sigid_coskew);
  lpost_samecoskew = -.5*(k-m)*log(2*pi) - sum(log(diag(chol(gam_delcovar)))) + ...
                     -.5*gam_coskewhat'*(gam_delcovar\gam_coskewhat);
        
 % same betai, i.e., each element of beta
  lpost_samebetai = log(normpdf(sparse(m+k,1),gamhat,sqrt(diag(gamvar))));        
    
   % add these to "storage loop"
    if loop > burnin
        i = loop - burnin;
        sbeta1(i,:) = beta1';
        sbeta2(i,:) = beta2';
        sSig1(i,:) = Sig1(Sigid)';
        sSig2(i,:) = Sig2(Sigid)';
        sCorr1(i,:) = corr1';
        sCorr2(i,:) = corr2';
        sS = sS + S;
        slpost_samebeta(i,:) = lpost_samebeta;
        slpost_samemu(i,:) = lpost_samemu;
        slpost_samedel(i,:) = lpost_samedel;
        slpost_samskew(i,:) = lpost_samskew;
        slpost_samecoskew(i,:) = lpost_samecoskew;
        slpost_samebetai(i,:) = lpost_samebetai;
     
    end
    
    if ( mod( loop, 1000 ) ==0 )
        disp(  [ num2str( loop ) ' loops... ' ] )        
    end
    
end
disp( ['MCMC takes '  num2str( etime( clock, start_time) ) ' seconds' ] );
disp(' ' );

disp('Computing BFs for various tests.... ');

%% Posterior means of the switching parameters
 n = 10; % use every nth draw
% covariance 
 Sig1hat = eye(m);
 Sig1hat(Sigid) = squeeze(mean(sSig1(1:n:end,:)));
 Sig2hat = eye(m);
 Sig2hat(Sigid) = squeeze(mean(sSig2(1:n:end,:)));
 Matrix_Sig1hat = (Sig1hat + Sig1hat') - triu(Sig1hat);
 Matrix_Sig2hat = (Sig2hat + Sig2hat') - triu(Sig2hat);
 Shat = sS/(nloop-burnin);
% mean, skewness and coskewness
 beta1hat = mean(sbeta1(1:n:end,:))';   beta2hat = mean(sbeta2(1:n:end,:))';
 beta1std = std(sbeta1(1:n:end,:))';    beta2std = std(sbeta2(1:n:end,:))';
 Del1hat = zeros(m,m);
 Del1hat(idDel) = beta1hat(m+1:end);
 Del1hat(idl) = Del1hat(idu);
 Del2hat = zeros(m,m);
 Del2hat(idDel) = beta2hat(m+1:end);
 Del2hat(idl) = Del2hat(idu);
% correlation 
 corr1hat = mean(sCorr1(1:n:end,:))'; corr2hat = mean(sCorr2(1:n:end,:))';

% %% Estimate the inefficiency factor
%  inefSig1 = ineff(sSig1(1:n:end,:));
%  inefSig2 = ineff(sSig2(1:n:end,:));
%  inefbeta1 = ineff(sbeta1(1:n:end,:));
%  inefbeta2 = ineff(sbeta2(1:n:end,:));
%  name = [repmat('beta1 ',m+k,1); repmat('beta2 ',m+k,1); repmat('Sigma1',k,1); repmat('Sigma2',k,1)];
%  figure; boxplot([inefbeta1; inefbeta2; inefSig1; inefSig2], name, 'whisker', 10);

%% compute the SD density ratio
% Structural break test (i) through mean  
  sslpost_samebetai = slpost_samebetai(1:n:end,:)'; 
  size_sslpost_samebetai = size(sslpost_samebetai,2); 
  pro_samebetai = (sum(sslpost_samebetai(1:m,:)' > zeros(size_sslpost_samebetai,m)))/(size_sslpost_samebetai);
 
% Structural break test (all i) through mean 
  sslpost_samemu = slpost_samemu(1:n:end);
  size_sslpost_samemu = size(sslpost_samemu,1);
  probability_samemu = (sum(sslpost_samemu > zeros(size_sslpost_samemu,1)))/(size_sslpost_samemu);

% Structural break test (i) through variance
  sSig = sSig1(1:n:end,:) - sSig2(1:n:end,:);
  size_sSig = size(sSig,1); 
  probability_sameSig = (sum(sSig < zeros(size_sSig,k)))/(size_sSig);
  probability_sameSig(kk_loc);
 
% Structural break test (all i) through variance
  store_ind_sig = sum(sSig(repmat(kk_loc',size_sSig,1)) < zeros(size_sSig,m),2); 
  pro_sameSig_g = sum(store_ind_sig == m*ones(size_sSig,1))/size_sSig;

% Structural break test (i) through skewness 
  max_samebetai = max(slpost_samebetai(1:n:end,:))';
  lpost_samebetai = log(mean(exp(slpost_samebetai(1:n:end,:)-repmat(max_samebetai',(nloop-burnin)/n,1))))' + max_samebetai;
  lpri_samebetai = log(normpdf(sparse(m+k,1),beta0,sqrt(diag(Vbeta))));  
  lBF_samebetai = lpost_samebetai - lpri_samebetai;
  lBF_sameskewcoskewi = lBF_samebetai(m+1:m+k);
  lBF_sameskewcoskewi(kk_loc);
 
% Structural break test (all i) through skewness 
  max_sameskew = max( slpost_samskew(1:n:end));
  lpost_sameskew = log(mean(exp(slpost_samskew(1:n:end)-max_sameskew))) + max_sameskew;
  Vbeta = invVbeta\speye(k+m);
  Vskew = eye(m); % Vbeta for skewness
   for i = 1:m
    Vskew(i,i) = Vbeta(kk_loc(i)+m,kk_loc(i)+m);
   end
  lpri_sameskew = log(mvnpdf(sparse(1,m),beta0((kk_loc+m)')',2*Vskew)); 
  lBF_sameskew = lpost_sameskew - lpri_sameskew;

% Structural break test (i) through mean, volatility and skewness
  kk = 10000; 
  IBF_muvolskewi = zeros(m,1);    
  for i = 1: m
   %posterior for sig_vol
    Sigpost1_vol = sSig1(1:n:end,kk_loc(i,1));
    Sigpost2_vol = sSig2(1:n:end,kk_loc(i,1));
    siglpost_difsig_vol = gaucopulanew060712(0,Sigpost1_vol-Sigpost2_vol);   
   %posterior for mu
    Mupost1 = sbeta1(1:n:end,i);
    Mupost2 = sbeta2(1:n:end,i);
    Mupost_difmu = gaucopulanew060712(0,Mupost1-Mupost2);   
   %posterior for skew
    Skewpost1 = sbeta1(1:n:end,m+kk_loc(i,1));
    Skewpost2 = sbeta2(1:n:end,m+kk_loc(i,1));
    Skewpost_difske = gaucopulanew060712(0,Skewpost1-Skewpost2);
   %prior for skew
    skewpri_rnd1 = zeros(kk,1); skewpri_rnd2 = zeros(kk,1);
    for j = 1: kk
      skewpri_rnd1(j,1) = normrnd(beta0(m+1,1),Vbeta(m+1,m+1));
      skewpri_rnd2(j,1) = normrnd(beta0(m+1,1),Vbeta(m+1,m+1));
    end
    skewpri = gaucopulanew060712(0,skewpri_rnd1-skewpri_rnd2);
   %prior for sig
    sigpri_rnd1 = zeros(kk,1); sigpri_rnd2 = zeros(kk,1);
    for j = 1: kk
      sigpri_rnd1(j,1) = iwishrnd(Vsig0(1,1),nuSig0);
      sigpri_rnd2(j,1) = iwishrnd(Vsig0(1,1),nuSig0);
    end
    sigpri = gaucopulanew060712(0,sigpri_rnd1-sigpri_rnd2); 
   %prior for mu
    mupri_rnd1 = zeros(kk,1); mupri_rnd2 = zeros(kk,1);
    for j = 1: kk
      mupri_rnd1(j,1) = normrnd(beta0(1,1),Vbeta(1,1));
      mupri_rnd2(j,1) = normrnd(beta0(1,1),Vbeta(1,1));
    end
    mupri = gaucopulanew060712(0,mupri_rnd1-mupri_rnd2);
    IBF_muvolskewi(i,1) = siglpost_difsig_vol + Mupost_difmu + Skewpost_difske - sigpri - mupri - skewpri;
  end
  
% Structural break test (all i) through mean, volatility and skewness
  lpri_samemu = log(mvnpdf(sparse(1,m),beta0(1:m)',2*Vbeta(1:m,1:m)));  
  Sigpost1_vol_all = sSig1(1:n:end,kk_loc'); %posterior
  Sigpost2_vol_all = sSig2(1:n:end,kk_loc');
  siglpost_difsig_vol_all = gaucopulanew060712(zeros(m,1)',Sigpost1_vol_all-Sigpost2_vol_all);
  sigpri_rnd_vol_all = zeros(kk,m); %prior 
   for j = 1: kk
      sig_rnd_all = iwishrnd(Vsig0,nuSig0);
      sigpri_rnd_vol_all(j,:) = diag(sig_rnd_all)';
   end
  sigpri_vol = gaucopulanew060712(zeros(m,1)',sigpri_rnd_vol_all);
  siglpost_sigs_vol_all = siglpost_difsig_vol_all - sigpri_vol;
  IBF_samemuvolskew = lpost_samemu -lpri_samemu + lpost_sameskew  - lpri_sameskew + siglpost_sigs_vol_all;
  
% Contagion test (i) through correlation
  sCorr = sCorr1(1:n:end,:) - sCorr2(1:n:end,:);
  size_sCorr = size(sCorr,1); 
  probability_sameScorr = (sum(sCorr < zeros(size_sCorr,k-m)))/(size_sCorr);
 
% Contagion test (all i) through correlation
  store_ind_corr = sum(sCorr(repmat(kk_loc',size_sCorr,1)) < zeros(size_sCorr,m),2);
  pro_sameCorr_g = sum(store_ind_corr == m*ones(size_sCorr,1))/size_sCorr;  
  
% Contagion test (i) through coskewness
  lBF_sameskewcoskewi_all = lBF_sameskewcoskewi(Vcoskew_loc);
  IBF_sameskewcoskewi_EU_US = lBF_sameskewcoskewi_all([4 7 9 10]');
 
% Contagion test (all i) through coskewness
  max_samecoskew = max(slpost_samecoskew(1:n:end));
  lpost_sameskew = log(mean(exp(slpost_samecoskew(1:n:end)-max_samecoskew))) + max_samecoskew;
  Vcoskew = eye(k-m); % Vbeta for coskewness
   for i = 1:k-m
    Vcoskew(i,i) = Vbeta(Vcoskew_loc(i)+m,Vcoskew_loc(i)+m);
   end
  lpri_samecoskew = log(mvnpdf(sparse(1,k-m),beta0(Vcoskew_loc+m)',2*Vcoskew)); 
  lBF_samecoskew = lpost_sameskew - lpri_samecoskew;  

% Contagion test (i) through correlation and coskewness
  k_corr = (m*(m-1))/2;
  IBF_Corrpost = zeros(k_corr,1);
  for i = 1: k_corr
    %posterior
     Corrpost1 = sCorr1(1:n:end,i);
     Corrpost2 = sCorr2(1:n:end,i);
     Corrpost_dif = gaucopulanew060712(0,Corrpost2-Corrpost1);
    %prior
     sigpri_rnd1 = zeros(kk,1);
     sigpri_rnd2 = zeros(kk,1);
     for j = 1: kk
      sigpri_rnd1(j,1) = iwishrnd(Vsig0(1,1),nuSig0);
      sigpri_rnd2(j,1) = iwishrnd(Vsig0(1,1),nuSig0);
     end
     sigpri = gaucopulanew060712(0,sigpri_rnd2-sigpri_rnd1);
     IBF_Corrpost(i,1) = Corrpost_dif - sigpri;
  end
  lBF_samecorrcoskei_all = IBF_Corrpost  + lBF_sameskewcoskewi_all;
  lBF_samecorrcoskei_EU_US = lBF_samecorrcoskei_all([4 7 9 10]');

% Contagion test (all i) through correlation and coskewness
  Corrpost1 = sCorr1(1:n:end,:);
  Corrpost2 = sCorr2(1:n:end,:);
  siglpost_difcorr_all = gaucopulanew060712(zeros(k-m,1)',Corrpost1-Corrpost2);
  pri_corr_rnd_all1 = zeros(kk,k-m); pri_corr_rnd_all2 = zeros(kk,k-m); %prior
   for j = 1: kk
      corr_rnd_all1 = iwishrnd(Vsig0,nuSig0);  corr_rnd_all2 = iwishrnd(Vsig0,nuSig0);
      store_corr1 = corr_rnd_all1(Sigid); store_corr2 = corr_rnd_all2(Sigid);
      pri_corr_rnd_all1(j,:) = store_corr1(Vcoskew_loc)';
      pri_corr_rnd_all2(j,:) = store_corr2(Vcoskew_loc)';
   end
  pri_corr = gaucopulanew060712(zeros(k-m,1)',pri_corr_rnd_all1-pri_corr_rnd_all2);
  IBF_corrlpost_all = siglpost_difcorr_all - pri_corr;
  IBF_samecorrcoske =  IBF_corrlpost_all + lBF_samecoskew;
  
% Joint contagion and structural break test (i)  
  IBF_mean_individual = lBF_samebetai(1:4); % mean 
  temp_IBF_skew_individual = lBF_sameskewcoskewi(kk_loc); 
  IBF__skew_individual = temp_IBF_skew_individual(1:4); % skewness 
  IBF_coskew_EU_US = IBF_sameskewcoskewi_EU_US; % coskewness
  IBF_sigi = zeros(k,1);
  for i = 1: k
    %posterior
     Sigpost1 = sSig1(1:n:end,i);
     Sigpost2 = sSig2(1:n:end,i);
     lpost_difsig = gaucopulanew060712(0,Sigpost1-Sigpost2);
    %prior
     sigpri_rnd1 = zeros(kk,1); sigpri_rnd2 = zeros(kk,1);
     for j = 1: kk
      sigpri_rnd1(j,1) = iwishrnd(Vsig0(1,1),nuSig0);
      sigpri_rnd2(j,1) = iwishrnd(Vsig0(1,1),nuSig0);
     end
     lpri_difsig = gaucopulanew060712(0,sigpri_rnd1-sigpri_rnd2);
     IBF_sigi(i,1) = lpost_difsig - lpri_difsig;
  end
  IBF_sigi_allcovar = IBF_sigi(Vcoskew_loc);
  IBF_sigi_EU_US = IBF_sigi_allcovar([4 7 9 10]');
  temp_IBF_vol_individual = IBF_sigi(kk_loc);
  IBF_vol_individual = temp_IBF_vol_individual(1:4); % volatility 
  IBF_covol_individual = IBF_sigi_EU_US; %corre channel 
  IBF_all_individual = IBF_mean_individual + IBF_vol_individual + IBF_covol_individual + IBF__skew_individual + IBF_coskew_EU_US;

% Joint contagion and structural break test (all i)  
  IBF_all = IBF_samemuvolskew + IBF_samecorrcoske;
 
disp('BFs computations done.');
disp('Contagion tests');
disp('The probability of correlation contagion between the US and France, Germany, Greece and Italy');
probability_sameScorr
disp('The probability of correlation contagion between the US and all European countries');
pro_sameCorr_g
disp('BFs for coskewness contagion between the US and France, Germany, Greece and Italy');
IBF_sameskewcoskewi_EU_US
disp('BFs for coskewness contagion between the US and all European countries');
lBF_samecoskew
disp('BFs for correlation and coskewness contagion between the US and France, Germany, Greece and Italy');
lBF_samecorrcoskei_EU_US
disp('BFs for correlation and coskewness contagion between the US and all European countries');
IBF_samecorrcoske 
disp('Structural break tests');
disp('The probability for France, Germany, Greece and Italy based on structural break test through mean');
probability_samemu
disp('The probability for all European countries based on structural break test through mean');
pro_samebetai
disp('The probability for France, Germany, Greece and Italy based on structural break test through variance');
probability_sameSig(kk_loc)
disp('The probability for all European countries based on structural break test through variance');
pro_sameSig_g
disp('BFs for France, Germany, Greece and Italy based on structural break test through skewness');
lBF_sameskewcoskewi(kk_loc)
disp('BFs for all European countries based on structural break test through skewness');
lBF_sameskew
disp('BFs for France, Germany, Greece and Italy based on structural break test through mean, variance and skewness');
IBF_muvolskewi
disp('BFs for all European countries based on structural break test through mean, variance and skewness');
IBF_samemuvolskew
disp('Joint contagion and structural break tests');
disp('BFs for joint contagion between the US and France, Germany, Greece and Italy');
IBF_all_individual
disp('BFs for joint contagion between the US and all European countries');
IBF_all
